package fumdb.domain;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public abstract class PersistentData {

	protected static final int NO_ID = 0;

	private Persistence persistence;
	
	public PersistentData(Persistence persistence) {
		this.persistence = persistence;
	}
	
	protected Persistence getPersistence() {
		return persistence;
	}
	
	public boolean commit() {
		if (isNew()) {
			return create();
		} else {
			return update();
		}
	}
	
	protected boolean create() {
		persistence.raiseError("Diese Funktionalitt ist leider nicht implementiert.");
		return false;
	}
	
	protected boolean update() {
		persistence.raiseError("Diese Funktionalitt ist leider nicht implementiert.");
		return false;
	}
	
	public boolean delete() {
		if (isNew()) {
			return true;
		}
		persistence.raiseError("Diese Funktionalitt ist leider nicht implementiert.");
		return false;
	}

	public boolean isNew() {
		return false;
	}
}