package fumdb.domain;

import java.sql.Date;
import java.util.Vector;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public class Person extends Musician {

	private String firstName;
	private String lastName;
	private Date birthday;
	private boolean isMale;

	public Person(Persistence persistence, int id) {
		super(persistence, id);
	}

	public Person(Persistence persistence) {
		this(persistence, NO_ID);
	}

	/**
	 * @see fumdb.domain.Musician#getName()
	 */
	public String getName() {
		if (firstName == null) {
			return lastName;
		} else { 
			return firstName + ' ' + lastName;
		}
	}

	/**
	 * @see fumdb.domain.Musician#isBand()
	 */
	public boolean isBand() {
		return false;
	}

	public Date getBirthday() {
		return birthday;
	}

	public String getFirstName() {
		return firstName;
	}

	public boolean isMale() {
		return isMale;
	}

	public String getLastName() {
		return lastName;
	}

	public void setBirthday(Date birthday) {
		this.birthday = birthday;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public void setMale(boolean isMale) {
		this.isMale = isMale;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public Vector getBandMemberships() {
		return getPersistence().getBandMemberships(getId());
	}

	public String toString() {
		String result = (getId() == NO_ID) ? "[unknown person] " : "[person " + getId() + "] ";
		result += getName() + " " + (isMale() ? "(m)" : "(w)");
		if (birthday != null) {
			result += " " + birthday;
		}
		return result;
	}
	
	protected boolean update() {
		return getPersistence().updatePerson(this);
	}
	
	protected boolean create() {
		return super.create(getPersistence().createPerson(this));
	}
}
