package fumdb.domain;

import java.sql.Date;
import java.util.Iterator;
import java.util.Vector;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public class Record extends PersistentData {

	private int id = NO_ID;
	private String title;
	private int musicianID = NO_ID;
	private String musicianName = null;
	private Date publishedDate = null;
	private String recordCompany = null;

	public Record(Persistence persistence, int id) {
		super(persistence);
		this.id = id;
	}

	public Record(Persistence persistence, Musician musician) {
		this(persistence, NO_ID);
		setMusician(musician);
	}

	public Record(Persistence persistence) {
		this(persistence, NO_ID);
		setMusicianToNull();
	}

	public String toString() {
		String result;
		if (getId() == NO_ID) {
			result = "[unsaved record] ";
		} else {
			result = "[record " + getId() + "] ";
		}
		if (hasMainArtist()) {
			result += "various";
		} else {
			result += "'" + getMusicianName() + "'";
		}
		result += " - '" + getTitle() + "'";
		if (getRecordCompanyName() != null) {
			result += ", " + getRecordCompanyName();
		}
		if (getPublishedDate() != null) {
			result += " (" + getPublishedDate() + ")";
		}
		return result;
	}

	public int getId() {
		return id;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getTitle() {
		return title;
	}

	public void setMusician(int musicianID, String musicianName) {
		this.musicianID = musicianID;
		this.musicianName = musicianName;
	}

	public void setMusician(Musician musician) {
		this.musicianID = musician.getId();
		this.musicianName = musician.getName();
	}

	public void setMusicianToNull() {
		this.musicianID = NO_ID;
		this.musicianName = null;
	}
	
	public int getMusicianID() {
		return musicianID;
	}

	public void setPublishedDate(Date publishedDate) {
		this.publishedDate = publishedDate;
	}

	public Date getPublishedDate() {
		return publishedDate;
	}

	public void setRecordCompany(String recordCompany) {
		this.recordCompany = recordCompany;
	}

	public String getRecordCompanyName() {
		return recordCompany;
	}
	
	public RecordCompany getRecordCompany() {
		if (getRecordCompanyName() == null) {
			return null;
		}
		Iterator it = getPersistence().getRecordCompanies().iterator();
		while (it.hasNext()) {
			RecordCompany each = (RecordCompany) it.next();
			if (each.getName().equals(getRecordCompanyName())) {
				return each;
			}
		}
		return null;
	}

	public boolean hasMainArtist() {
		return musicianID != NO_ID;
	}

	public String getMusicianName() {
		if (hasMainArtist()) {
			return musicianName;
		} else {
			return "various";
		}
	}
	
	public String getFullTitle() {
		return getMusicianName() + " - " + getTitle();
	}

	public Musician getMusician() {
		return getPersistence().getMusician(getMusicianID());
	}

	public Vector getAllFunctions() {
		return getPersistence().getAllFunctionsByRecord(getId());
	}

	public Tracklist getTracklist() {
		return getPersistence().getTracklist(getId());
	}
	
	public boolean isNew() {
		return id == NO_ID;
	}

	protected boolean create() {
		int id = getPersistence().createRecord(this);
		if (id == NO_ID) {
			return false;
		}
		this.id = id;
		return true;
	}

	protected boolean update() {
		return getPersistence().updateRecord(this);
	}

	public boolean delete() {
		if (isNew()) {
			return true;
		}
		return getPersistence().deleteRecord(getId());
	}
}
