package fumdb.domain;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public class Song extends PersistentData {

	private SongId id;
	private String name;
	private String genre;
	private SongId original;

	public Song(Persistence persistence, SongId id) {
		super(persistence);
		this.id = id;
	}
	
	public Song(Persistence persistence, Musician performer) {
		this(persistence, new SongId(performer.getId()));
	}

	public String getGenreName() {
		return genre;
	}

	public Genre getGenre() {
		return getPersistence().getGenres().get(genre);
	}

	public String getName() {
		return name;
	}

	public Song getOriginal() {
		return getPersistence().getSong(original);
	}

	public SongId getOriginalId() {
		return original;
	}

	public void setGenreName(String genreName) {
		this.genre = genreName;
	}

	public void setGenre(Genre genre) {
		this.genre = genre.getName();
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setOriginal(Song original) {
		this.original = original.getId();
	}

	public void setOriginal(SongId original) {
		this.original = original;
	}

	public SongId getId() {
		return id;
	}
	
	public int getMusicianId() {
		return id.getMusicianId();
	}
	
	public int getSongId() {
		return id.getSongId();
	}
	
	public Musician getMusician() {
		return getPersistence().getMusician(getMusicianId());
	}
	
	public boolean isNew() {
		return id.isUnknown();
	}

	protected boolean update() {
		return getPersistence().updateSong(this);
	}

	protected boolean create() {
		int newId = getPersistence().createSong(this);
		if (newId == NO_ID) {
			return false;
		}
		id.setSongId(newId);
		return true;
	}

	public boolean delete() {
		if (isNew()) {
			return true;
		}
		return getPersistence().deleteSong(id);
	}
	
	public String toString() {
		return "[song] musician: " + getMusicianId() + ", songId: " + getSongId() + " '" + getName() + "'";
	}
}
