package fumdb.domain;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public class Tracklist extends PersistentData {

	private int recordId;
	private Map tracks = new HashMap();
	private Map lengths = new HashMap();
	private int max = 0;

	public Tracklist(Persistence persistence, int recordId) {
		super(persistence);
		this.recordId = recordId;
	}
	
	public void addSong(int trackId, Song song, int length) {
		Integer track = new Integer(trackId);
		tracks.put(track, song);
		lengths.put(track, new Integer(length));
		max = Math.max(max, trackId);
	}

	public Song getSong(int trackId) {
		return (Song) tracks.get(new Integer(trackId));
	}
	
	public int getLength(int trackId) {
		Integer result = (Integer) lengths.get(new Integer(trackId));
		if (result == null) return 0;
		return result.intValue();
	}
	
	public int getRecordId() {
		return recordId;
	}

	public Record getRecord() {
		return getPersistence().getRecord(recordId);
	}
	
	public boolean isCompilation() {
		Iterator it = tracks.values().iterator();
		int musician = ((Song) it.next()).getMusicianId();
		while (it.hasNext()) {
			int eachMusician = ((Song) it.next()).getMusicianId();
			if (eachMusician != musician) {
				return true;
			}
		}
		return false;
	}
	
	public int size() {
		return max;
	}
	
	public Iterator iterator() {
		return tracks.values().iterator();
	}

	private int getGenreCount(String genreName) {
		int result = 0;
		Iterator it = iterator();
		while (it.hasNext()) {
			Song each = (Song) it.next();
			if (genreName.equals(each.getGenreName())) {
				result++;
			}
		}
		return result;
	}

	public Vector getGenres() {
		Vector result = new Vector();
		Iterator it = iterator();
		while (it.hasNext()) {
			String genreName = ((Song) it.next()).getGenreName();
			if (result.contains(genreName) || genreName == null) continue;
			int count = getGenreCount(genreName);
			int insertAt = 0;
			for (int i = 0; i < result.size(); i++) {
				if (getGenreCount((String) result.get(i)) < count) break;
				insertAt++;
			}
			result.insertElementAt(genreName, insertAt);
		}
		return result;
	}
	
	public int getLength() {
		Iterator it = lengths.values().iterator();
		int result = 0;
		while (it.hasNext()) {
			Integer each = (Integer) it.next();
			result += each.intValue();
		}
		return result;
	}
}
