package fumdb.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import fumdb.domain.Band;
import fumdb.domain.Member;
import fumdb.domain.Person;
import fumdb.domain.Record;

/**
 * @author anja
 */

public class FormViewBand extends FormView implements ActionListener {
	FuMuDbGUI gui;
	Band band;
	Vector recordsByBand = new Vector();
	Vector bandMemberships = new Vector();
	JPanel form = new JPanel();
		
	JLabel labelName = new JLabel("Name : ");
	JLabel labelURL = new JLabel("URL : ");
	JLabel labelMembers = new JLabel("Mitglieder / Knstler : ");
	JLabel labelRecords = new JLabel("verffentlichte Tontrger : ");

	DefaultListModel artistNameListModel = new DefaultListModel();
	JList artistNameList = new JList(artistNameListModel);

	JTextField fieldName = new JTextField(30);
	JTextField fieldURL = new JTextField(40);

	JButton buttonAddMember = new JButton("Mitglied hinzufgen");
	JButton buttonDeleteMember = new JButton("Mitglied lschen");
	JButton buttonDeleteBand = new JButton("Band lschen");
	JButton buttonAddBand = new JButton("Band speichern");

	DefaultListModel recordListModel = new DefaultListModel();
	JList recordList = new JList(recordListModel);
	
	public FormViewBand(FuMuDbGUI gui, Dimension size, String title) {
		super(size, title);
		this.gui = gui;

		createFrame();
		createForm();
	}
	
	public FormViewBand(FuMuDbGUI gui, String bandName, Band band) {
		super(gui.getDesktopDimension(), "Band/Knstlername : "+bandName);
		this.gui = gui;
		this.band = band;
		createFrame();

		fieldName.setText(band.getName());
		fieldURL.setText(band.getURL());
		
		Vector records = gui.getPersistence().getRecordsByMusician(band.getId());
		for (int i=0; i<records.size(); i++) {
			recordsByBand.add(((Record) records.elementAt(i)).getTitle());
		}

		Vector memberships = band.getMembers();
		for (int i=0; i<memberships.size(); i++) {
			Person person = (Person) ((Member) memberships.elementAt(i)).getPerson();
			bandMemberships.add(person.getName());
		}

		createForm();
	}
	
	public void createForm() {
 		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosed(InternalFrameEvent e) {
				gui.deleteInternalFrame(FormViewBand.this);
			}
		});

		artistNameList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		recordList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		if (bandMemberships.size()>0) {
			for (int i=0; i<bandMemberships.size(); i++) {
				artistNameListModel.add(i, bandMemberships.elementAt(i));
			}
		}
		
		if (recordsByBand.size()>0) {
			for (int i=0; i<recordsByBand.size(); i++) {
				recordListModel.add(i, recordsByBand.elementAt(i));
			}
		}

		recordList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				int index = recordList.getSelectedIndex();
				if (index != -1) {
					Vector records = new Vector();
					Vector recordVec = gui.getPersistence().getRecordsByMusician(band.getId());
					for (int i=0; i<recordVec.size(); i++) {
						records.add(((Record) recordVec.elementAt(i)).getTitle());
					}
					if (records.contains(recordList.getSelectedValue().toString())) {
						int ind = records.indexOf(recordList.getSelectedValue().toString());
						FormViewRecord frame = new FormViewRecord(gui, recordList.getSelectedValue().toString(), (Record) recordVec.elementAt(ind));
						gui.addInternalFrameToVector(frame, recordList.getSelectedValue().toString());
						gui.repaint();
					}
				}
			}
		});

		buttonAddMember.addActionListener(this);
		buttonDeleteMember.addActionListener(this);
		buttonAddBand.addActionListener(this);
		buttonDeleteBand.addActionListener(this);
		
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		form.setLayout(gridbag);
		
		gbc.weightx = 0;
		gbc.weighty = 0.5;
		
		addToGBL(form, gridbag, labelName, 0, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldName, 1, 1, 6, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, labelURL, 0, 2, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldURL, 1, 2, 6, 1, GridBagConstraints.HORIZONTAL);
             
		addToGBL(form, gridbag, labelMembers, 0, 3, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, artistNameList, 1, 3, 4, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonAddMember, 6, 3, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonDeleteMember, 6, 4, 1, 1, GridBagConstraints.HORIZONTAL);
             
		addToGBL(form, gridbag, labelRecords, 0, 5, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, recordList, 1, 5, 5, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, buttonDeleteBand, 6, 7, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonAddBand, 6, 8, 1, 1, GridBagConstraints.HORIZONTAL);

		getContentPane().add(form);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == buttonAddMember) {
			ListDialog addBandDialog = new ListDialog(this, gui, "Musiker hinzufgen", true, gui.getPersistence().getAllPersonNames() , gui.MUSICIANS);
			return;
		}
		if (e.getSource() == buttonDeleteMember) {
			int index = artistNameList.getSelectedIndex();
			if (index != -1 && artistNameListModel.size()>0) {
				artistNameListModel.remove(index);
			}
			return;
		}
		if (e.getSource() == buttonAddBand) {
			if (!fieldName.getText().equals("")) {
				if (band == null) {
					band = new Band(gui.getPersistence());
					band.setName(fieldName.getText());
					band.setURL(fieldURL.getText());
					boolean commited = band.commit();
					if (commited) {
						this.dispose();
						gui.deleteInternalFrame(this);
					}
				} else {
					band.setName(fieldName.getText());
					band.setURL(fieldURL.getText());
					boolean commited = band.commit();
					if (commited) {
						this.dispose();
						gui.deleteInternalFrame(this);
					}
				}	
			}	else {
				JOptionPane.showMessageDialog(this, "Name unvollstndig");
			}
			return;
		}
		if (e.getSource() == buttonDeleteBand) {
			if (band != null) {
				boolean deleted = band.delete();
				if (deleted) {
					this.dispose();
					gui.deleteInternalFrame(this);
				}
			}
			return;
		}
	}
	
	public void lookUpMusician() {
	}
	
	public ImageIcon getIcon() {
		return new ImageIcon(getClass().getResource("artist_icon_add.gif").getFile());
	}
}
