package fumdb.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.sql.Date;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import fumdb.domain.Band;
import fumdb.domain.Member;
import fumdb.domain.Person;
import fumdb.domain.Record;

/**
 * @author anja
 */

public class FormViewPerson extends FormView implements ActionListener {
	FuMuDbGUI gui;
	Person person = null;
	Vector originalMemberships;
	Vector newMemberships = new Vector();
	Map bandNames;
	boolean isMale = true;
	Vector recordsByPerson = new Vector();
	Vector bandMemberships = new Vector();
	
	JPanel form = new JPanel();
		
	JLabel labelFirstName = new JLabel("Vorname : ");
	JLabel labelLastName = new JLabel("Nachname : ");
	JLabel labelBirthday = new JLabel("Geburtstag : ");
	JLabel labelSex = new JLabel("Geschlecht : ");
	JLabel labelURL = new JLabel("URL : ");

	JLabel labelMemberOf = new JLabel("Bandmitglied bei : ");
	JLabel labelArtistName = new JLabel("Knstlername(n) : ");
	JLabel labelRecords = new JLabel("verffentlichte Tontrger : ");

	DefaultListModel bandListModel = new DefaultListModel();
	JList bandList = new JList(bandListModel);

	DefaultListModel artistNameListModel = new DefaultListModel();
	JList artistNameList = new JList(artistNameListModel);

	DefaultListModel recordListModel = new DefaultListModel();
	JList recordList = new JList(recordListModel);

	JTextField fieldFirstName = new JTextField(30);
	JTextField fieldLastName = new JTextField(30);

	JTextField fieldBirthDay = new JTextField(2);
	JTextField fieldBirthMonth = new JTextField(2);
	JTextField fieldBirthYear = new JTextField(4);

	JTextField fieldURL = new JTextField(40);

	JButton buttonAddBand = new JButton("Band hinzufgen");
	JButton buttonDeleteBand = new JButton("Band lschen");

	JButton buttonAddArtistName = new JButton("Knstlernamen hinzufgen");
	JButton buttonDeleteArtistName = new JButton("Knstlernamen lschen");
	
	JButton buttonDeletePerson = new JButton("Person lschen");
	JButton buttonAddPerson = new JButton("Person speichern");

	String[] personSex = {"mnnlich", "weiblich"};
	JComboBox comboSex;
	
	
	public FormViewPerson(FuMuDbGUI gui, Dimension size, String title) {
		super(size, title);
		this.gui = gui;

		createFrame();
		createForm();
	}

	public FormViewPerson(FuMuDbGUI gui, String personName, Person person) {
		super(gui.getDesktopDimension(), "Musiker : "+personName);
		this.gui = gui;
		this.person = person;
		createFrame();
		
		fieldFirstName.setText(person.getFirstName());
		fieldLastName.setText(person.getLastName());
		if (person.getBirthday() != null) {
			fieldBirthYear.setText(person.getBirthday().toString().substring(0,4));
			fieldBirthMonth.setText(person.getBirthday().toString().substring(5,7));
			fieldBirthDay.setText(person.getBirthday().toString().substring(8,10));
		}
		isMale = person.isMale();
		fieldURL.setText(person.getURL());
		
		Vector records = gui.getPersistence().getRecordsByMusician(person.getId());
		for (int i=0; i<records.size(); i++) {
			recordsByPerson.add(((Record) records.elementAt(i)).getTitle());
		}

		originalMemberships = person.getBandMemberships();
		for (int i=0; i<originalMemberships.size(); i++) {
			Band band = (Band) ((Member) originalMemberships.elementAt(i)).getBand();
			bandMemberships.add(band.getName());
		}
		
		createForm();
	}
	
	public void createForm() {
 		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosed(InternalFrameEvent e) {
				gui.deleteInternalFrame(FormViewPerson.this);
			}
		});

		comboSex = new JComboBox(personSex);
		if (isMale) {
			comboSex.setSelectedIndex(0);
		} else {
			comboSex.setSelectedIndex(1);
		}

		bandList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		artistNameList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		recordList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		recordList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				int index = recordList.getSelectedIndex();
				if (index != -1) {
					Vector records = new Vector();
					Vector originalRecords = gui.getPersistence().getRecordsByMusician(person.getId());
					for (int i=0; i<originalRecords.size(); i++) {
						records.add(((Record) originalRecords.elementAt(i)).getTitle());
					}
					if (records.contains(recordList.getSelectedValue().toString())) {
						int ind = records.indexOf(recordList.getSelectedValue().toString());
						FormViewRecord frame = new FormViewRecord(gui, recordList.getSelectedValue().toString(), (Record) originalRecords.elementAt(ind));
						gui.addInternalFrameToVector(frame, recordList.getSelectedValue().toString());
						gui.repaint();
					}
				}
			}
		});
		
		buttonAddBand.addActionListener(this);
		buttonDeleteBand.addActionListener(this);
		buttonAddArtistName.addActionListener(this);
		buttonDeleteArtistName.addActionListener(this);
		buttonDeletePerson.addActionListener(this);
		buttonAddPerson.addActionListener(this);

		Border blackline = BorderFactory.createEtchedBorder();
		bandList.setBorder(blackline);
		artistNameList.setBorder(blackline);
		recordList.setBorder(blackline);

		if (bandMemberships.size()>0) {
			for (int i=0; i<bandMemberships.size(); i++) {
				bandListModel.add(i, bandMemberships.elementAt(i));
			}
		}
		
		if (recordsByPerson.size()>0) {
			for (int i=0; i<recordsByPerson.size(); i++) {
				recordListModel.add(i, recordsByPerson.elementAt(i));
			}
		}
		
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		form.setLayout(gridbag);
		
		gbc.weightx = 0;
		gbc.weighty = 0.5;
		
		addToGBL(form, gridbag, labelFirstName, 0, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldFirstName, 1, 1, 6, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, labelLastName, 0, 2, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldLastName, 1, 2, 6, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, labelBirthday, 0, 3, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldBirthDay, 1, 3, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldBirthMonth, 2, 3, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldBirthYear, 3, 3, 1, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, labelSex, 0, 4, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, comboSex, 1, 4, 3, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, labelURL, 0, 5, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, fieldURL, 1, 5, 6, 1, GridBagConstraints.HORIZONTAL);
             
		addToGBL(form, gridbag, labelMemberOf, 0, 7, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, bandList, 1, 7, 4, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonAddBand, 6, 7, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonDeleteBand, 6, 8, 1, 1, GridBagConstraints.HORIZONTAL);
             
		addToGBL(form, gridbag, labelArtistName, 0, 13, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, artistNameList, 1, 13, 4, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonAddArtistName, 6, 13, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonDeleteArtistName, 6, 14, 1, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, labelRecords, 0, 16, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, recordList, 1, 16, 4, 1, GridBagConstraints.HORIZONTAL);

		addToGBL(form, gridbag, buttonDeletePerson, 6, 18, 1, 1, GridBagConstraints.HORIZONTAL);
		addToGBL(form, gridbag, buttonAddPerson, 6, 19, 1, 1, GridBagConstraints.HORIZONTAL);

		getContentPane().add(form);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == buttonAddBand) {
			bandNames = gui.getPersistence().getAllBandNames();
			ListDialog addBandDialog = new ListDialog(this, gui, "Band hinzufgen", true, bandNames, gui.BANDS);
			return;
		}
		if (e.getSource() == buttonDeleteBand) {
			int index = bandList.getSelectedIndex();
			if (index != -1 && bandListModel.size()>0) {
				if (person == null) {
					bandListModel.remove(index);
				} else {
					Member membership = (Member) originalMemberships.elementAt(index);
					boolean deleted = membership.delete();
					if (deleted) {
						originalMemberships.remove(index);
						bandListModel.remove(index);
						if (newMemberships.contains(((Band) membership.getBand()).getName())) {
							int index1 = newMemberships.indexOf(((Band) membership.getBand()).getName());
							newMemberships.remove(index1);
						}
					}
				}				
			}
			return;
		}
		if (e.getSource() == buttonAddArtistName) {
			bandNames = gui.getPersistence().getAllBandNames();
			ListDialog addBandDialog = new ListDialog(this, gui, "Knstlernamen hinzufgen", true, bandNames, gui.ARTIST_NAME);
			return;
		}
		if (e.getSource() == buttonDeleteArtistName) {
			int index = artistNameList.getSelectedIndex();
			if (index != -1 && artistNameListModel.size()>0) {
				artistNameListModel.remove(index);
			}
			return;
		}
		if (e.getSource() == buttonDeletePerson) {
			if (person != null) {
				boolean deleted = person.delete();
				if (deleted) {
					this.dispose();
					gui.deleteInternalFrame(this);
				}
			}
			return;
		}
		if (e.getSource() == buttonAddPerson) {
			if (!fieldFirstName.getText().equals("") && !fieldLastName.getText().equals("")) {
				if (person == null) {
					person = new Person(gui.getPersistence());
					person.setFirstName(fieldFirstName.getText());
					person.setLastName(fieldLastName.getText());
					person.setURL(fieldURL.getText());
					if (!fieldBirthYear.getText().equals("")) {
						try {
							if (fieldBirthDay.getText().equals("")) {
								fieldBirthDay.setText("01");
							}	
							if (fieldBirthMonth.getText().equals("")) {
								fieldBirthMonth.setText("01");
							}	
							Date birthday = Date.valueOf(fieldBirthYear.getText()+"-"+fieldBirthMonth.getText()+"-"+fieldBirthDay.getText());
							person.setBirthday(birthday);
						} catch (IllegalArgumentException exc) {
							JOptionPane.showMessageDialog(this, "Falsches Datumsformat. Richtiges Format : DD MM YYYY");
						}
					}
					if (comboSex.getSelectedIndex() == 0) {
						person.setMale(true);
					} else {
						person.setMale(false);
					}
					boolean commited = person.commit();
					if (commited) {
	/*					for (int i=0; i<newMemberships.size(); i++) {
							int bandID	= 0;
							Iterator it = bandNames.keySet().iterator();
							while (it.hasNext()) {
					   			if (((String) it.next()).equals(newMemberships.elementAt(i).toString())) {
						   			bandID = Integer.parseInt(bandNames.get(newMemberships.elementAt(i).toString()).toString());
					   			}
							}
							if (bandID != 0) {
								Member member = new Member(gui.getPersistence(), person.getId(), bandID);
								boolean memberCommited = member.commit();
								if (!commited) {
									JOptionPane.showMessageDialog(this, "Fehler beim Speichern der Daten");
									return;
								}
							} else {
								JOptionPane.showMessageDialog(this, "Fehler beim Speichern der Daten");
								return;
							}
						}
	*/					
						this.dispose();
						gui.deleteInternalFrame(this);
					}
				} else {
					person.setFirstName(fieldFirstName.getText());
					person.setLastName(fieldLastName.getText());
					person.setURL(fieldURL.getText());
					if (!fieldBirthYear.getText().equals("")) {
						try {
							if (fieldBirthDay.getText().equals("")) {
								fieldBirthDay.setText("01");
							}	
							if (fieldBirthMonth.getText().equals("")) {
								fieldBirthMonth.setText("01");
							}	
							Date birthday = Date.valueOf(fieldBirthYear.getText()+"-"+fieldBirthMonth.getText()+"-"+fieldBirthDay.getText());
							person.setBirthday(birthday);
						} catch (IllegalArgumentException exc) {
							JOptionPane.showMessageDialog(this, "Falsches Datumsformat. Richtiges Format : DD MM YYYY");
						}
					}
					if (comboSex.getSelectedIndex() == 0) {
						person.setMale(true);
					} else {
						person.setMale(false);
					}
					boolean commited = person.commit();
					if (commited) {
	/*					for (int i=0; i<newMemberships.size(); i++) {
							int bandID	= 0;
							Iterator it = bandNames.keySet().iterator();
							while (it.hasNext()) {
					   			if (((String) it.next()).equals(newMemberships.elementAt(i).toString())) {
						   			bandID = Integer.parseInt(bandNames.get(newMemberships.elementAt(i).toString()).toString());
					   			}
							}
							if (bandID != 0) {
								Member member = new Member(gui.getPersistence(), person.getId(), bandID);
								boolean memberCommited = member.commit();
								if (!commited) {
									JOptionPane.showMessageDialog(this, "Fehler beim Speichern der Daten");
									return;
								}
							} else {
								JOptionPane.showMessageDialog(this, "Fehler beim Speichern der Daten");
								return;
							}
						}
	*/					
						this.dispose();
						gui.deleteInternalFrame(this);
					}
				}	
			}	else {
				JOptionPane.showMessageDialog(this, "Name unvollstndig");
			}
			return;
		}
	}
	
	public ImageIcon getIcon() {
		return new ImageIcon(getClass().getResource("artist_icon_add.gif").getFile());
	}
}
