package fumdb.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import fumdb.sql.Persistence;
import fumdb.sql.SQLErrorListener;

/**
 * @author anja
 */
public class FuMuDbGUI extends JFrame implements ActionListener {
	final static int STANDARD_VIEW = 0;
	final static int PERSONAL_VIEW = 1;

	final static int BANDS = 0;	
	final static int MUSICIANS = 1;
	final static int BANDS_MUSICIANS = 2;
	final static int ARTIST_NAME = 3;

	private Persistence db;

	public Vector internalFrameVector = new Vector();
	public Vector personalViewLinks = new Vector();
	
	Font titleFont = new Font("Verdana", Font.PLAIN, 9);
	JDesktopPane desktop;
	int frameNumber = 0; 

	ImageIcon song = new ImageIcon(getClass().getResource("song_icon.gif").getFile());
	ImageIcon record = new ImageIcon(getClass().getResource("record_icon.gif").getFile());
	ImageIcon artist = new ImageIcon(getClass().getResource("artist_icon.gif").getFile());
	
	JPanel screenPanel = new JPanel();
	JPanel viewPanel = new JPanel();
	DefaultListModel viewModel = new DefaultListModel();
	DefaultListModel standardViewModel = new DefaultListModel();
	JList standardViewList = new JList(standardViewModel);
	JList viewList = new JList(viewModel);

	JToolBar toolBar = new JToolBar();
	JButton button;
	JButton button1;
	JButton button2;
	JButton button3;
	JButton button4;
	JButton button5;
	JButton button6;
	JButton button7;
	JButton button8;
	JButton button9;
	
	public FuMuDbGUI() throws SQLException {
		super("FUMuDB");

		try {
			db = new Persistence(new SimpleErrorListener());
			db.connect();

			createFrame();
		} catch (ClassNotFoundException e) {
			System.out.println("Unable to locate Oracle driver");
			System.out.println("Please make sure that oracle.jdbc.driver.OracleDriver is on the CLASSPATH!");
		}

	}

	public void createFrame() {
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent evt){
				try {
					db.disconnect();
				} catch (SQLException e) {
					e.printStackTrace();
				}
				dispose();
			}
		});
 
		desktop = new JDesktopPane(); 
		desktop.setBackground(Color.WHITE);
		
		screenPanel.setLayout(new BorderLayout());
		viewPanel.setLayout(new BorderLayout());
		
		button = new JButton(new ImageIcon(getClass().getResource("record_add.gif").getFile()));
		button1 = new JButton(new ImageIcon(getClass().getResource("artist_add.gif").getFile()));
		button2 = new JButton(new ImageIcon(getClass().getResource("artist_add.gif").getFile()));
		button3 = new JButton(new ImageIcon(getClass().getResource("song_add.gif").getFile()));
		
		button4 = new JButton(new ImageIcon(getClass().getResource("record.gif").getFile()));
		button5 = new JButton(new ImageIcon(getClass().getResource("artist.gif").getFile()));
		button6 = new JButton(new ImageIcon(getClass().getResource("artist.gif").getFile()));
		button7 = new JButton(new ImageIcon(getClass().getResource("song.gif").getFile()));
		button8 = new JButton(new ImageIcon(getClass().getResource("search.gif").getFile()));
		button9 = new JButton(new ImageIcon(getClass().getResource("query.gif").getFile()));
		
		button.addActionListener(this);
		button1.addActionListener(this);
		button2.addActionListener(this);
//		button3.addActionListener(this);
		button4.addActionListener(this);
		button5.addActionListener(this);
		button6.addActionListener(this);
//		button7.addActionListener(this);
		button8.addActionListener(this);
//		button9.addActionListener(this);
		
		button.setToolTipText("neuer Tontrger");
		button1.setToolTipText("neuer Musiker");
		button2.setToolTipText("neue Band/neuer Knstlername");
//		button3.setToolTipText("neuer Song");
		button4.setToolTipText("Liste aller Tontrger");
		button5.setToolTipText("Liste aller Musiker");
		button6.setToolTipText("Liste aller Bands/Knstlernamen");
//		button7.setToolTipText("Liste aller Songs");
		button8.setToolTipText("Suche");
//		button9.setToolTipText("Anfrage");
		toolBar.add(button);
		toolBar.add(button1);
		toolBar.add(button2);
//		toolBar.add(button3);
		toolBar.addSeparator();
		toolBar.add(button4);
		toolBar.add(button5);
		toolBar.add(button6);
//		toolBar.add(button7);
		toolBar.add(button8);
		toolBar.setFloatable(false);
		
		FUMuDBListRenderer standardRenderer = new FUMuDBListRenderer(standardViewList, this);
		FUMuDBListRenderer viewRenderer = new FUMuDBListRenderer(viewList, this);
		standardViewList.setCellRenderer(standardRenderer);
		viewList.setCellRenderer(viewRenderer);
		
		viewList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		standardViewList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
//		standardViewModel.add(0,"Anfrage");
		standardViewModel.add(0,"Suche");
//		standardViewModel.add(0,"alle Songs");
		standardViewModel.add(0,"alle Bands");
		standardViewModel.add(0,"alle Musiker");
		standardViewModel.add(0,"alle Tontrger");
//		standardViewModel.add(0,"neuer Song");
		standardViewModel.add(0,"neue Band");
		standardViewModel.add(0,"neuer Musiker");
		standardViewModel.add(0,"neuer Tontrger");
		
		standardViewList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				actionPerformed(STANDARD_VIEW);
			}
		});
		viewList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				actionPerformed(PERSONAL_VIEW);
			}
		});
		
		Border blackline = BorderFactory.createLineBorder(Color.black);
		
		TitledBorder standardLine = BorderFactory.createTitledBorder(blackline, "Standard Views", TitledBorder.CENTER, TitledBorder.TOP);
		standardLine.setTitleFont(titleFont);
		standardViewList.setBorder(standardLine);
		
		TitledBorder viewLine = BorderFactory.createTitledBorder(blackline, "Personal Views", TitledBorder.CENTER, TitledBorder.TOP);
		viewLine.setTitleFont(titleFont);
		viewList.setBorder(viewLine);
		
		viewList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		viewPanel.add(standardViewList, BorderLayout.NORTH);
		viewPanel.add(viewList, BorderLayout.CENTER);
		
		desktop.setBorder(blackline);
		
		screenPanel.add(toolBar, BorderLayout.NORTH);
		viewPanel.setPreferredSize(new Dimension(130,viewPanel.getHeight()));
		viewPanel.setMaximumSize(new Dimension(130,viewPanel.getHeight()));
		viewPanel.setMinimumSize(new Dimension(130,viewPanel.getHeight()));
		screenPanel.add(viewPanel, BorderLayout.WEST);
		screenPanel.add(desktop, BorderLayout.CENTER);
		
		getContentPane().add(screenPanel);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation((screenSize.width - 800)/2,(screenSize.height - 600)/2);
		this.setSize(800,600);
		this.setVisible(true);
		this.setResizable(false);
		this.validate();
	}

	public void actionPerformed(int view) {
		if (view == STANDARD_VIEW) {
			int index = standardViewList.getSelectedIndex();
			Dimension frameSize = new Dimension(desktop.getWidth(),desktop.getHeight());
			String title = standardViewModel.getElementAt(index).toString();

			JInternalFrame frame = new JInternalFrame(); 
			
			if (title.indexOf("Tontrger") != -1) {
				if (title.toString().indexOf("alle") != -1) {
					frame = new ListViewRecord(this, frameSize, "alle Tontger"); 
					addInternalFrameToVector(frame,"alle Tontrger");
				} else {
					frame = new FormViewRecord(this, frameSize, title); 
					addInternalFrameToVector(frame,title);
				}
			}
			if (title.indexOf("Musiker") != -1) {
				if (title.toString().indexOf("alle") != -1) {
					frame = new ListViewPerson(this, frameSize, "alle Musiker"); 
					addInternalFrameToVector(frame,"alle Musiker");
				} else {
					frame = new FormViewPerson(this, frameSize, title); 
					addInternalFrameToVector(frame,title);
				}
			}
			if (title.indexOf("Band") != -1) {
				if (title.toString().indexOf("alle") != -1) {
					frame = new ListViewBand(this, frameSize, "alle Bands / Knstlernamen"); 
					addInternalFrameToVector(frame,"alle Bands");
				} else {
					frame = new FormViewBand(this, frameSize, title); 
					addInternalFrameToVector(frame,title);
				}
			}
			if (title.indexOf("Song") != -1) {
/*				if (title.toString().indexOf("alle") != -1) {
					frame = new ListViewSong(this, frameSize, "alle Songs"); 
					addInternalFrameToVector(frame,"alle Songs");
				} else {
					frame = new FormViewSong(this, frameSize, title); 
					addInternalFrameToVector(frame,title);
				}
*/			}
			if (title.toString().indexOf("Suche") != -1) {
				FormSearch search = new FormSearch(this, "Suche", true); 
			}

			repaint();
			return;
		}
		if (view == PERSONAL_VIEW) {
			int index = viewList.getSelectedIndex();
			showPersonalView(index);
			return;
		}
		repaint(); 
	}

	public void actionPerformed(ActionEvent e) {
		Dimension frameSize = getDesktopDimension();
		String title = "";
		JInternalFrame frame = new JInternalFrame(); 
					
		if (e.getSource() == button) {
			frame = new FormViewRecord(this, frameSize, "neuer Tontrger"); 
			addInternalFrameToVector(frame, "neuer Tontrger");
		}
		if (e.getSource() == button1) {
			frame = new FormViewPerson(this, frameSize, "neuer Musiker"); 
			addInternalFrameToVector(frame, "neuer Musiker");
		}
		if (e.getSource() == button2) {
			frame = new FormViewBand(this, frameSize, "neue Band / neuer Knstlername"); 
			addInternalFrameToVector(frame, "neue Band");
		}
		if (e.getSource() == button3) {
//			frame = new FormViewSong(this, frameSize, "neuer Song"); 
//			addInternalFrameToVector(frame, "neuer Song");
		}
		if (e.getSource() == button4) {
			frame = new ListViewRecord(this, frameSize, "alle Tontrger"); 
			addInternalFrameToVector(frame, "alle Tontrger");
		}
		if (e.getSource() == button5) {
			frame = new ListViewPerson(this, frameSize, "alle Musiker"); 
			addInternalFrameToVector(frame, "alle Musiker");
		}
		if (e.getSource() == button6) {
			frame = new ListViewBand(this, frameSize, "alle Bands/Knstlernamen"); 
			addInternalFrameToVector(frame, "alle Bands");
		}
		if (e.getSource() == button7) {
//			frame = new ListViewSong(this, frameSize, "alle Songs"); 
		}
		if (e.getSource() == button8) {
			FormSearch search = new FormSearch(this, "Suche", true); 
		}
		return;
	}

	public Dimension getDesktopDimension() {
		Dimension frameSize = new Dimension(desktop.getWidth(),desktop.getHeight());
		return frameSize;
	}

	public Persistence getPersistence() {
		return db;
	}
	
	public void addInternalFrameToVector(JInternalFrame frame, String title) {
		internalFrameVector.add(0, frame);
		personalViewLinks.add(0, title);
		viewModel.add(0, title);
		viewList.setSelectedIndex(0);
		desktop.add(frame); 
		frame.moveToFront(); 
		frame.setVisible(true);
		repaint();
	}

	public void showPersonalView(int index) {
		((JInternalFrame) internalFrameVector.elementAt(index)).moveToFront(); 
	}

	public void deleteInternalFrame(JInternalFrame frame) {
		int index = internalFrameVector.indexOf(frame);
		if (index != -1) {
			internalFrameVector.remove(index);
			personalViewLinks.remove(index);
			viewModel.remove(index);
			repaint();
		}
	}

class SimpleErrorListener implements SQLErrorListener {
	public void catchSQLException(SQLException e) {
		e.printStackTrace();
		JOptionPane.showMessageDialog(FuMuDbGUI.this, e.toString());
	}
}	
}

