package fumdb.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import fumdb.domain.Band;
import fumdb.domain.Member;
import fumdb.domain.Musician;
import fumdb.domain.Person;

/**
 * @author anja
 */

public class ListViewBand extends ListView implements ActionListener {
	FuMuDbGUI gui;
	Vector bands = new Vector();
	
	JPanel screenPanel = new JPanel();
	JPanel bandList = new JPanel();
	JScrollPane scrollPane;
	JPanel buttonPanel = new JPanel();

	JButton showBand = new JButton("Band/Knstlername zeigen / editieren");
	JButton deleteBand = new JButton("Band/Knstlername lschen");

	JTable bandListTable;
	FuMuDbTableModel bandModel = new FuMuDbTableModel();

	public ListViewBand(FuMuDbGUI gui, Dimension size, String title) {
		super(size, title);
		this.gui = gui;
		createFrame();
		createList();
	}
	
	public void createList() {
 		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosed(InternalFrameEvent e) {
				gui.deleteInternalFrame(ListViewBand.this);
			}
		});

		Vector musicians = gui.getPersistence().getAllMusicians();
   		for (int i=0; i<musicians.size(); i++) {
   			Musician musician = (Musician) musicians.elementAt(i);
   			if (musician.isBand()) {
	   			bands.add(musician);
   			}
		}

		bandListTable = new JTable(bandModel);		
		bandModel.addColumn("Name");
		bandModel.addColumn("richtiger Name / Mitglieder");
		bandModel.addColumn("URL");
		bandListTable.getTableHeader().setReorderingAllowed(false);
	    bandListTable.getTableHeader().setResizingAllowed(false);
		bandListTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		bandListTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
			}
		});

		for (int i=0; i<bands.size(); i++) {
			Band band = (Band) bands.elementAt(i);
			Vector memberships = band.getMembers();
			String memberNames = "";
			for (int j=0; j<memberships.size(); j++) {
				Person memberInfos = ((Member) memberships.get(j)).getPerson();
				memberNames += memberInfos.getName();
				if (j < memberships.size()-1) {
					memberNames += ", ";
				}
			}
			bandModel.addRow(new Object[]{ band.getName(), memberNames, band.getURL()});
		}

		bandList.setLayout(new BorderLayout());
		bandList.add(bandListTable.getTableHeader(), BorderLayout.NORTH);
		bandList.add(bandListTable, BorderLayout.CENTER);

		scrollPane = new JScrollPane(bandList);
		screenPanel.setLayout(new BorderLayout());
		screenPanel.add(scrollPane, BorderLayout.CENTER);
		
		showBand.addActionListener(this);
		deleteBand.addActionListener(this);
		
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(showBand);
		buttonPanel.add(deleteBand);
		
		screenPanel.add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(screenPanel);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == showBand) {
			int index = bandListTable.getSelectedRow();
			if (index != -1) {
				String bandName = ((Band) bands.elementAt(index)).getName();
				FormViewBand frame = new FormViewBand(gui, bandName, (Band) bands.elementAt(index));
				gui.addInternalFrameToVector(frame, bandName);
				gui.repaint();
			}
			return;
		}
		if (e.getSource() == deleteBand) {
			int index = bandListTable.getSelectedRow();
			if (index != -1) {
				if (bands != null) {
					Band band = (Band) bands.elementAt(index);
					boolean deleted = band.delete();
					if (deleted) {
						bands.remove(index);
						bandModel.removeRow(index);
						repaint();
					} else {
						JOptionPane.showMessageDialog(this, "Fehler beim Lschen der Band");
					}
				}
			}
			return;
		}
	}
	
	public ImageIcon getIcon() {
		return new ImageIcon(getClass().getResource("artist_icon.gif").getFile());
	}
}
