package fumdb.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import fumdb.domain.Band;
import fumdb.domain.Musician;
import fumdb.domain.Person;

/**
 * @author anja
 */

public class ListViewMusician extends ListView implements ActionListener {
	FuMuDbGUI gui;
	Vector musicians = new Vector();
	
	JPanel screenPanel = new JPanel();
	JPanel bandList = new JPanel();
	JScrollPane scrollPane;
	JPanel buttonPanel = new JPanel();

	JButton showBand = new JButton("Band/Musiker zeigen / editieren");
	JButton deleteBand = new JButton("Band/Musiker lschen");

	JTable bandListTable;
	FuMuDbTableModel bandModel = new FuMuDbTableModel();

	public ListViewMusician(FuMuDbGUI gui, Dimension size, String title, Vector musicians) {
		super(size, title);
		this.gui = gui;
		this.musicians = musicians;
		createFrame();
		createList();
	}
	
	public void createList() {
 		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosed(InternalFrameEvent e) {
				gui.deleteInternalFrame(ListViewMusician.this);
			}
		});

		bandListTable = new JTable(bandModel);		
		bandModel.addColumn("Name");
		bandModel.addColumn("URL");
		bandModel.addColumn("Musiker / Band");
		bandListTable.getTableHeader().setReorderingAllowed(false);
	    bandListTable.getTableHeader().setResizingAllowed(false);
		bandListTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		for (int i=0; i<musicians.size(); i++) {
			Musician musician = (Musician) musicians.elementAt(i);
			if (musician.isBand()) {
				Band band = (Band) musician;
				bandModel.addRow(new Object[]{ band.getName(), band.getURL(), "Band"});
			} else {
				Person person = (Person) musician;
				bandModel.addRow(new Object[]{ person.getName(), person.getURL(), "Musiker"});
			}
		}

		bandList.setLayout(new BorderLayout());
		bandList.add(bandListTable.getTableHeader(), BorderLayout.NORTH);
		bandList.add(bandListTable, BorderLayout.CENTER);

		scrollPane = new JScrollPane(bandList);
		screenPanel.setLayout(new BorderLayout());
		screenPanel.add(scrollPane, BorderLayout.CENTER);
		
		showBand.addActionListener(this);
		deleteBand.addActionListener(this);
		
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(showBand);
		buttonPanel.add(deleteBand);
		
		screenPanel.add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(screenPanel);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == showBand) {
			int index = bandListTable.getSelectedRow();
			if (index != -1) {
				Musician musician = (Musician) musicians.elementAt(index);
				if (musician.isBand()) {
					String bandName = ((Band) musician).getName();
					FormViewBand frame = new FormViewBand(gui, bandName, (Band) musician);
					gui.addInternalFrameToVector(frame, bandName);
					gui.repaint();
				} else {
					String personName = ((Person) musician).getName();
					FormViewPerson frame = new FormViewPerson(gui, personName, (Person) musician);
					gui.addInternalFrameToVector(frame, personName);
					gui.repaint();
				}
			}
			return;
		}
		if (e.getSource() == deleteBand) {
			int index = bandListTable.getSelectedRow();
			if (index != -1) {
				if (musicians != null) {
					Musician musician = (Musician) musicians.elementAt(index);
					boolean deleted = musician.delete();
					if (deleted) {
						musicians.remove(index);
						bandModel.removeRow(index);
						repaint();
					} else {
						JOptionPane.showMessageDialog(this, "Fehler beim Lschen des Musikers");
					}
				}
			}
			return;
		}
	}
	
	public ImageIcon getIcon() {
		return new ImageIcon(getClass().getResource("artist_icon.gif").getFile());
	}
}
