package fumdb.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import fumdb.domain.Band;
import fumdb.domain.Member;
import fumdb.domain.Musician;
import fumdb.domain.Person;

/**
 * @author anja
 */

public class ListViewPerson extends ListView implements ActionListener {
	FuMuDbGUI gui;
	Vector persons = new Vector();
	
	JPanel screenPanel = new JPanel();
	JPanel artistList = new JPanel();
	JScrollPane scrollPane;
	JPanel buttonPanel = new JPanel();

	JButton showPerson = new JButton("Musiker zeigen / editieren");
	JButton deletePerson = new JButton("Musiker lschen");

	JTable artistListTable;
	FuMuDbTableModel artistModel = new FuMuDbTableModel();

	public ListViewPerson(FuMuDbGUI gui, Dimension size, String title) {
		super(size, title);
		this.gui = gui;
		createFrame();
		createList();
	}
	
	public void createList() {
 		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosed(InternalFrameEvent e) {
				gui.deleteInternalFrame(ListViewPerson.this);
			}
		});

		Vector musicians = gui.getPersistence().getAllMusicians();
   		for (int i=0; i<musicians.size(); i++) {
   			Musician musician = (Musician) musicians.elementAt(i);
   			if (!musician.isBand()) {
	   			persons.add(musician);
   			}
		}

		artistListTable = new JTable(artistModel);		
		artistModel.addColumn("Name");
		artistModel.addColumn("Geburtsdatum");
		artistModel.addColumn("Geschlecht");
		artistModel.addColumn("URL");
		artistModel.addColumn("Bands/Knstlernamen");

		artistListTable.getTableHeader().setReorderingAllowed(false);
	    artistListTable.getTableHeader().setResizingAllowed(false);
		artistListTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		artistListTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
			}
		});
		
		for (int i=0; i<persons.size(); i++) {
			Person person = (Person) persons.elementAt(i);
			Vector memberships = person.getBandMemberships();
			String memberNames = "";
			for (int j=0; j<memberships.size(); j++) {
				Band memberInfos = ((Member) memberships.get(j)).getBand();
				memberNames += memberInfos.getName();
				if (j < memberships.size()-1) {
					memberNames += ", ";
				}
			}
			String birthday = "";
			if (person.getBirthday() != null) {
				birthday = person.getBirthday().toString();
			}
			String sex = "weiblich";
			if (person.isMale()) {
				sex = "mnnlich";
			}

			artistModel.addRow(new Object[]{person.getName(), birthday, sex, person.getURL(), memberNames});
		}
		
		artistList.setLayout(new BorderLayout());
		artistList.add(artistListTable.getTableHeader(), BorderLayout.NORTH);
		scrollPane = new JScrollPane(artistListTable);
		artistList.add(scrollPane, BorderLayout.CENTER);

		artistList.setLayout(new BorderLayout());
		artistList.add(artistListTable.getTableHeader(), BorderLayout.NORTH);
		artistList.add(artistListTable, BorderLayout.CENTER);

		scrollPane = new JScrollPane(artistList);
		screenPanel.setLayout(new BorderLayout());
		screenPanel.add(scrollPane, BorderLayout.CENTER);
		
		showPerson.addActionListener(this);
		deletePerson.addActionListener(this);
		
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(showPerson);
		buttonPanel.add(deletePerson);
		
		screenPanel.add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(screenPanel);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == showPerson) {
			int index = artistListTable.getSelectedRow();
			if (index != -1) {
				String personName = ((Person) persons.elementAt(index)).getName();
				FormViewPerson frame = new FormViewPerson(gui, personName, (Person) persons.elementAt(index));
				gui.addInternalFrameToVector(frame, personName);
				gui.repaint();
			}
			return;
		}
		if (e.getSource() == deletePerson) {
			int index = artistListTable.getSelectedRow();
			if (index != -1) {
				if (persons != null) {
					Person person = (Person) persons.elementAt(index);
					boolean deleted = person.delete();
					if (deleted) {
						persons.remove(index);
						artistModel.removeRow(index);
						repaint();
					} else {
						JOptionPane.showMessageDialog(this, "Fehler beim Lschen des Musikers");
					}
				}
			}
			return;
		}
	}
	
	public ImageIcon getIcon() {
		return new ImageIcon(getClass().getResource("artist_icon.gif").getFile());
	}
}
