package fumdb.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import fumdb.domain.Record;

/**
 * @author anja
 */

public class ListViewRecord extends ListView implements ActionListener {
	FuMuDbGUI gui;
	Vector recordVector = null;
	JPanel screenPanel = new JPanel();
	JPanel list = new JPanel();
	JScrollPane scrollPane;
	JPanel buttonPanel = new JPanel();
		
	JTable listTable;
	FuMuDbTableModel model = new FuMuDbTableModel();

	JButton showRecord = new JButton("Tontrger zeigen / editieren");
	JButton deleteRecord = new JButton("Tontrger lschen");
	
	Vector records;
		
	public ListViewRecord(FuMuDbGUI gui, Dimension size, String title) {
		super(size, title);
		this.gui = gui;
		createFrame();
		
		createList();
	}
	
	public ListViewRecord(FuMuDbGUI gui, Dimension size, String title, Vector recordVector) {
		super(size, title);
		this.gui = gui;
		this.recordVector = recordVector;
		createFrame();
		
		createList();
	}

	public void createList() {
 		this.addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosed(InternalFrameEvent e) {
				gui.deleteInternalFrame(ListViewRecord.this);
			}
		});

		listTable = new JTable(model);		

		model.addColumn("Titel");
		model.addColumn("Musiker / Band");
		model.addColumn("Erscheinungsdatum");
		model.addColumn("Plattenfirma");

		listTable.getTableHeader().setReorderingAllowed(false);
	    listTable.getTableHeader().setResizingAllowed(false);
		listTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		if (recordVector == null) {
			records = gui.getPersistence().getAllRecords();
	   		for (int i=0; i<records.size(); i++) {
	   			Record record = (Record) records.elementAt(i);
		   		model.addRow(new Object[]{record.getTitle(),record.getMusicianName(),record.getPublishedDate(),record.getRecordCompanyName()});
			}
		} else {
	   		for (int i=0; i<recordVector.size(); i++) {
	   			Record record = (Record) recordVector.elementAt(i);
		   		model.addRow(new Object[]{record.getTitle(),record.getMusicianName(),record.getPublishedDate(),record.getRecordCompanyName()});
			}
		}
				   		
		list.setLayout(new BorderLayout());
		list.add(listTable.getTableHeader(), BorderLayout.NORTH);
		list.add(listTable, BorderLayout.CENTER);
		
		scrollPane = new JScrollPane(list);
		screenPanel.setLayout(new BorderLayout());
		screenPanel.add(scrollPane, BorderLayout.CENTER);
		
		showRecord.addActionListener(this);
		deleteRecord.addActionListener(this);
		
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(showRecord);
		buttonPanel.add(deleteRecord);
		
		screenPanel.add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(screenPanel);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == showRecord) {
			int index = listTable.getSelectedRow();
			if (index != -1) {
				if (recordVector == null) {
					String recordTitle = ((Record) records.elementAt(index)).getTitle();
					FormViewRecord frame = new FormViewRecord(gui, recordTitle, (Record) records.elementAt(index));
					gui.addInternalFrameToVector(frame, recordTitle);
					gui.repaint();
				} else {
					String recordTitle = ((Record) recordVector.elementAt(index)).getTitle();
					FormViewRecord frame = new FormViewRecord(gui, recordTitle, (Record) recordVector.elementAt(index));
					gui.addInternalFrameToVector(frame, recordTitle);
					gui.repaint();
				}
			}
			return;	
		}
		if (e.getSource() == deleteRecord) {
			int index = listTable.getSelectedRow();
			if (index != -1) {
				if (recordVector != null) {
					Record record = (Record) recordVector.elementAt(index);
					boolean deleted = record.delete();
					if (!deleted) {
						JOptionPane.showMessageDialog(this, "Fehler beim Lschen des Tontrgers");
					} else {
						recordVector.remove(index);
						model.removeRow(index);
						repaint();
					}
				} else {
					Record record = (Record) records.elementAt(index);
					boolean deleted = record.delete();
					if (!deleted) {
						JOptionPane.showMessageDialog(this, "Fehler beim Lschen des Tontrgers");
					} else {
						records.remove(index);
						model.removeRow(index);
						repaint();
					}
				}
			}
			return;
		}
	}
	
	public ImageIcon getIcon() {
		return new ImageIcon(getClass().getResource("record_icon.gif").getFile());
	}
}
