package fumdb.servlet;

import java.io.PrintWriter;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public abstract class DatabasePage extends HtmlPage {

	private Persistence db;
	private ServletErrorListener errorListener;

	public void setDb(Persistence db, ServletErrorListener errorListener) {
		this.db = db;
		this.errorListener = errorListener;
	}

	protected Persistence getDb() {
		return db;
	}

	public void send(PrintWriter out) {
		try {
			init();
		} catch (RequestException e) {
			HtmlPage page = new RequestErrorPage(e.getMessage());
			page.send(out);
			return;
		}
		if (errorListener.isError()) return;
		super.send(out);
	}

	protected abstract void init() throws RequestException;
	protected abstract String getTitle();
	protected abstract String getHeadline();
	protected abstract void printBody();
	
	protected String getGenreLink(String genreName) {
		return link("genre=" + urlencode(genreName), genreName);
	}

	protected String getMusicianLink(String musicianName, int musicianId) {
		return link("mid=" + musicianId, musicianName);
	}
}
