package fumdb.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.SQLException;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import fumdb.sql.Persistence;

/**
 * @author Richard Cyganiak
 */
public class FUMuDBServlet extends HttpServlet {

	private Persistence db;
	private ServletErrorListener errorListener;

	public void init() {
		System.out.println("*** FUMuDB Servlet ***");
		System.out.println("Connecting to Oracle...");
		try {
			errorListener = new ServletErrorListener();
			db = new Persistence(errorListener);
			db.connect();
			System.out.println("...Done!");
		} catch (ClassNotFoundException e) {
			System.out.println("Unable to locate Oracle driver");
			System.out.println("Please make sure that oracle.jdbc.driver.OracleDriver is on the CLASSPATH!");
		} catch (SQLException e) {
			System.out.println("Unable to connect to database");
			System.out.println("Reported error: " + e.getMessage());
		}
	}

	/**
	 * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {

		errorListener.setPrintWriter(response.getWriter());
		response.setContentType("text/html");

		DatabasePage page;
		if (request.getParameter("genre") != null) {
			page = new GenrePage(request.getParameter("genre"));
		} else if (request.getParameter("tid") != null) {
			page = new RecordPage(request.getParameter("tid"));
		} else if (request.getParameter("mid") != null) {
			page = new MusicianPage(request.getParameter("mid"));
		} else if (request.getParameter("search") != null) {
			String category = request.getParameter("kategorie");
			if (category == null) {
				page = new HomePage();
			} else if (category.equals("m")) {
				page = null;
				page = new SearchMusicianPage(request.getParameter("search"));
			} else if (category.equals("t")) {
				page = new SearchRecordPage(request.getParameter("search"));
			} else {
				page = new HomePage();
			}
		} else {
			page = new HomePage();
		}
		errorListener.setError(false);
		page.setDb(db, errorListener);
		page.send(response.getWriter());
	}

	/**
	 * @see javax.servlet.Servlet#destroy()
	 */
	public void destroy() {
		try {
			db.disconnect();
		} catch (SQLException e) {
			System.out.println("Error while disconnecting");
			System.out.println("Reported error: " + e.getMessage());
		}
		super.destroy();
	}

}