package fumdb.servlet;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import fumdb.domain.Genre;
import fumdb.domain.Musician;
import fumdb.domain.Record;

/**
 * @author Richard Cyganiak
 */
public class GenrePage extends DatabasePage {

	private String genreName;
	private Vector musicians;
	private Map records;

	public GenrePage(String genre) {
		this.genreName = genre;
	}

	protected void init() throws RequestException {
		Genre g = getDb().getGenres().get(genreName);
		if (g == null) {
			throw new RequestException("Unbekanntes Genre: '" + genreName + "'");
		}
		musicians = getDb().getMusiciansByGenre(genreName);
		records = getDb().getRecordsByGenre(genreName);
	}

	protected String getTitle() {
		return "Genres: " + genreName;
	}

	protected String getHeadline() {
		return genreName;
	}

	protected void printBody() {
		if (musicians.size() > 0) {
			printMusicians();
		}
		if (records.size() > 0) {
			printRecords();
		}
		if (musicians.size() == 0 && records.size() == 0) {
			printEmpty();
		}
	}

	private void printMusicians() {
		h2("Knstler und Bands");
		Iterator it = musicians.iterator();
		out("<p>");
		while (it.hasNext()) {
			out(getMusicianHtml((Musician) it.next()));
		}
		out("</p>");
	}
	
	private String getMusicianHtml(Musician musician) {
		return link("mid=" + musician.getId(), musician.getName()) + "<br>";
	}
	
	private void printRecords() {
		h2("Tontrger");
		Iterator it = records.keySet().iterator();
		out("<p>");
		while (it.hasNext()) {
			Record each = (Record) it.next();
			Integer genreQuota = (Integer) records.get(each);
			out(getRecordHtml(each, genreQuota.intValue()));
		}
		out("</p>");
	}

	private String getRecordHtml(Record record, int genreQuota) {
		return link("tid=" + record.getId(), record.getFullTitle()) + " (" + genreQuota + "%)<br>";
	}

	private void printEmpty() {
		out("In diesem Genre haben wir keine Eintrge.");
	}
}