package fumdb.servlet;

import java.net.URLEncoder;
import java.util.Iterator;
import java.util.Map;

import fumdb.domain.Genre;
import fumdb.domain.Genres;

/**
 * @author Richard Cyganiak
 */
public class HomePage extends DatabasePage {

	Genres genres;
	Map genreCounts;

	protected void init() {
		genres = getDb().getGenres();
		genreCounts = getDb().getSongCountsByGenre();
	}

	protected String getTitle() {
		return "Homepage";
	}

	protected String getHeadline() {
		return "FUMuDB - die Internet-Musikdatenbank";
	}

	protected void printBody() {
		h2("Suche");
		out("<form>");
		out("<input type=\"text\" size=\"30\" name=\"search\"> ");
		out("<select name=\"kategorie\">");
		out("<option value=\"m\" selected>Musiker</option>");
		out("<option value=\"t\">Tontrger</option>");
		out("</select> ");
		out("<input type=\"submit\" name=\"submit\" value=\"Suchen\"></form>");
		h2("Genres");
		Iterator it = genres.getTopGenres().iterator();
		while (it.hasNext()) {
			Genre each = (Genre) it.next();
			printGenre(each, 0);
		}
	}
	
	private void printGenre(Genre genre, int level) {
		out(getGenreHtml(genre.getName(), getSongCount(genre.getName()), level));
		Iterator it = genre.getSubGenres().iterator();
		while (it.hasNext()) {
			Genre each = (Genre) it.next();
			printGenre(each, level + 1);
		}
	}

	private String getGenreHtml(String name, int songs, int level) {
		if (level == 0) {
			return "<p class=\"level0\">"
					+ "<strong>" + getGenreLink(name) + "</strong>"
					+ " (" + songs + ")</p>";
		} else {
			return "<p class=\"level" + level + "\">"
					+ getGenreLink(name)
					+ " (" + songs + ")</p>";
		}
	}

	private int getSongCount(String genreName) {
		Integer count = (Integer) genreCounts.get(genreName);
		if (count == null) return 0;
		return count.intValue();
	}
}