package fumdb.servlet;

import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

/**
 * @author Richard Cyganiak
 */
public abstract class HtmlPage {

	private PrintWriter out;

	public void send(PrintWriter out) {
		this.out = out;
		printHeader();
		printHeadline();
		printBody();
		printFooter();
	}

	protected abstract String getTitle();
	protected abstract String getHeadline();
	protected abstract void printBody();

	protected void out(String html) {
		out.println(html);
	}

	private String getFUMuDBTitle(String title) {
		if (title == null || title.equals("")) {
			return "FUMuDB";
		} else {
			return "FUMuDB - " + title;
		}
	}

	private void printHeader() {
		out("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">");
		out("<html>");
		out("<head>");
		out("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">");
		out("<title>" + getFUMuDBTitle(getTitle()) + "</title>");
		out("<link rel=\"stylesheet\" type=\"text/css\" href=\"styles.css\">");
		out("</head>");
		out("<body>");
	}
	
	private void printHeadline() {
		out("<h1>" + getHeadline() + "</h1>");
	}

	private void printFooter() {
		out("</body>");
		out("</html>");
	}
	
	protected void h2(String h2) {
		tag("h2", h2);
	}
	
	protected void p(String p) {
		tag("p", p);
	}

	protected String link(String target, String html) {
		if (target == null || "".equals(target)) {
			return "<a href=\"servlet\">" + html + "</a>";
		} else {
			return "<a href=\"servlet?" + target + "\">" + html + "</a>";
		}
	}

	protected String xlink(String target) {
		if (target.indexOf(':') == -1) {
			return "<a href=\"http://" + target + "\">http://" + target + "</a>";
		} else {
			return "<a href=\"" + target + "\">" + target + "</a>";
		}
	}

	protected void tag(String tag, String content) {
		out("<" + tag + ">" + content + "</" + tag + ">");
	}
	
	protected String urlencode(String s) {
		try {
			return URLEncoder.encode(s, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return "UnsupportedEncodingException :-(((";
		}
	}
}
