package fumdb.servlet;

import java.sql.Date;
import java.text.DateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import fumdb.domain.Band;
import fumdb.domain.Member;
import fumdb.domain.Musician;
import fumdb.domain.Person;
import fumdb.domain.Record;

/**
 * @author Richard Cyganiak
 */
public class MusicianPage extends DatabasePage {

	private String requestedMusician;
	private Musician musician;
	private Vector bands;
	private Vector members;
	private Map musicians;
	private Date birthday;
	private boolean isMale;
	private Vector records;

	public MusicianPage(String requestedMusician) {
		this.requestedMusician = requestedMusician;
	}

	protected void init() throws RequestException {
		try {
			int id = Integer.parseInt(requestedMusician);
			musician = getDb().getMusician(id);
		} catch (NumberFormatException e) {
			throw new RequestException("Unbekannte Knstler-ID: " + requestedMusician);
		}
		if (musician == null) {
			throw new RequestException("Unbekannte Knstler-ID: '" + requestedMusician);
		}
		if (musician.isBand()) {
			members = ((Band) musician).getMembers();
		} else {
			bands = ((Person) musician).getBandMemberships();
			birthday = ((Person) musician).getBirthday();
			isMale = ((Person) musician).isMale();
		}
		musicians = new HashMap();
//		musicians.put(new Integer()

		records	= musician.getRecords();
		
	}



	protected String getTitle() {
		return "Knstler: " + musician.getName();
	}

	protected String getHeadline() {
		return musician.getName();
	}

	protected void printBody() {
		boolean anythingPrinted = false;
		if (musician.getName() != null) {
			printMusicianName();
			anythingPrinted = true;
		}
		if (!musician.isBand()) {
			if (birthday != null) {
				printBirthday();
				anythingPrinted = true;
			}
			printSex(isMale);
			anythingPrinted = true;
			if (bands != null) {
				printBands();
				anythingPrinted = true;
			}
		} else {
			printMembers();
			anythingPrinted = true;
		}
		if (records != null && records.size()>0) {
			printRecords();
			anythingPrinted = true;
		}
		if (!anythingPrinted) {
			p("Keine weiteren Informationen vorhanden.");
		}
	}

	protected void printMusicianName() {
		p("Name:  <strong>"+ link("mid=" + musician.getId(), musician.getName()) + "</strong>");
	}

	protected void printBirthday() {
		DateFormat format = DateFormat.getDateInstance(DateFormat.LONG);
		String formattedDate = format.format(birthday);
		p("Geburtstag: "+formattedDate);
	}
	
	protected void printSex(boolean isMale) {
		if (isMale) {
			p("Geschlecht: mnnlich");
		} else {
			p("Geschlecht: weiblich");
		}	
	}
	
	protected void printBands() {
		p("Bandmitgliedschaften / Knstlernamen: ");
		for (int i=0; i<bands.size(); i++) {
			Band band = ((Member) bands.elementAt(i)).getBand();
			out(getMusicianHtml(band));
		}
	}

	protected void printMembers() {
		p("Bandmitglieder: ");
		for (int i=0; i<members.size(); i++) {
			Person person = ((Member) members.elementAt(i)).getPerson();
			out(getMusicianHtml(person));
		}
	}
	
	private String getMusicianHtml(Musician musician) {
		return link("mid=" + musician.getId(), musician.getName()) + "<br>";
	}
	
	
	protected void printRecords() {
		p("verffentlichte Tontrger: ");
		for (int i=0; i<records.size(); i++) {
			out(getRecordHtml((Record) records.elementAt(i)));
		}
	}
	
	private String getRecordHtml(Record record) {
		return link("tid=" + record.getId(), record.getTitle())+"<br>";
	}
}