package fumdb.servlet;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import fumdb.domain.Genre;
import fumdb.domain.Musician;
import fumdb.domain.Record;

/**
 * @author Richard Cyganiak
 */
public class SearchMusicianPage extends DatabasePage {

	private String search;
	private Vector musicians = new Vector();
	private Vector urls = new Vector();

	public SearchMusicianPage(String search) {
		this.search = search;
	}

	protected void init() throws RequestException {
		musicians = getDb().getSimilarMusicians(search);
		if (musicians == null) {
			throw new RequestException("Fehler bei Suche");
		}
		
		for (int i=0; i<musicians.size(); i++) {
			urls.add(((Musician) musicians.elementAt(i)).getURL());
		}
	}

	protected String getTitle() {
		return "Suche: Musiker - " + search;
	}

	protected String getHeadline() {
		return "Suche: Musiker - "+search;
	}

	protected void printBody() {
		if (musicians.size() == 0) {
			printEmpty();
		} else {
			out("<p>");
			out("<table cellspacing=\"5\" cellpadding=\"5\"><tr><td><b>Musikername<br></b></td><td><b>URL<br></b></td></tr>");
			for (int i=0; i<musicians.size(); i++) {
				out("<tr>");
				printMusician(i);
				printURL(i);
				out("</tr>");
			}
			out("</table>");
			out("</p>");
		}
	}

	private void printMusician(int index) {
		out("<td>");
		if (musicians.elementAt(index) != null) {
			out(getMusicianHtml((Musician) musicians.elementAt(index)));
		}
		out("</td>");
	}


	private String getMusicianHtml(Musician musician) {
		return link("mid=" + musician.getId(), musician.getName()) + "<br>";
	}

	private void printURL(int index) {
		out("<td>");
		if (urls.elementAt(index) != null) {
			out(getUrlHtml(urls.elementAt(index).toString()));
		}
		out("</td>");
	}

	private String getUrlHtml(String url) {
		return xlink(url);
	}

	private void printEmpty() {
		out("Suche war nicht erfolgreich.");
	}
}