package fumdb.servlet;

import java.util.Vector;

import fumdb.domain.Musician;
import fumdb.domain.Record;

/**
 * @author Richard Cyganiak
 */
public class SearchRecordPage extends DatabasePage {

	private String search;
	private Vector records = new Vector();
	private Vector musicians = new Vector();
	private Vector recordCompanies = new Vector();

	public SearchRecordPage(String search) {
		this.search = search;
	}

	protected void init() throws RequestException {
		records = getDb().getSimilarRecords(search);
		if (records == null) {
			throw new RequestException("Fehler bei Suche");
		}
		
		for (int i=0; i<records.size(); i++) {
			musicians.add(((Record) records.elementAt(i)).getMusician());
			recordCompanies.add(((Record) records.elementAt(i)).getRecordCompanyName());
		}
	}

	protected String getTitle() {
		return "Suche: Tontrger - " + search;
	}

	protected String getHeadline() {
		return "Suche: Tontrger - "+search;
	}

	protected void printBody() {
		if (records.size() == 0) {
			printEmpty();
		} else {
			out("<p>");
			out("<table cellspacing=\"5\" cellpadding=\"5\"><tr><td><b>Tontrgertitel<br></b></td><td><b>Musiker<br></b></td><td><b>Plattenfirma<br></b></td></tr>");
			for (int i=0; i<records.size(); i++) {
				out("<tr>");
				printTitle(i);
				printMusician(i);
				printRecordCompany(i);
				out("</tr>");
			}
			out("</table>");
			out("</p>");
		}
	}

	private void printTitle(int index) {
		out("<td>");
		out(getRecordHtml((Record) records.elementAt(index)));
		out("</td>");
	}

	private void printMusician(int index) {
		out("<td>");
		if (musicians.elementAt(index) != null) {
			out(getMusicianHtml((Musician) musicians.elementAt(index)));
		}
		out("</td>");
	}


	private String getMusicianHtml(Musician musician) {
		return link("mid=" + musician.getId(), musician.getName()) + "<br>";
	}

	private void printRecordCompany(int index) {
		out("<td>");
		if (recordCompanies.elementAt(index) != null) {
			out(recordCompanies.elementAt(index).toString());
		}
		out("</td>");
	}

	private String getRecordHtml(Record record) {
		return link("tid=" + record.getId(), record.getTitle()) + "<br>";
	}

	private void printEmpty() {
		out("Suche war nicht erfolgreich.");
	}
}